import { queryFiatList, queryTokenUsdPrice, reqAlchemyPrePay, verifyApplePay, verifyGooglePay } from '../api/PayApi';
import { ZError } from '../common/ZError';
import { VERSION_CODE } from '../config/constants';
import { singleton } from '../decorator/singleton.decorator';
import { IPayData } from '../types/data.types';
import { NativeSvr } from './NativeSvr';

@singleton
export class PaySvr {
  private priceMap: Map<string, string> = new Map();
  /**
   * Calls the alchemyPrePay function with the given data.
   * @param data - The data to be passed to the alchemyPrePay function.
   * @returns The result of the alchemyPrePay function.
   */
  public async alchemyPrePay(data: any) {
    // data.network = data.network || jc.wallet.currentChain.network;
    // data.crypto = data.crypto.toUpperCase();
    // data.country = data.country.toUpperCase();
    // data.fiat = data.fiat.toUpperCase();
    data.evn = jc.wallet.env;
    data.version = VERSION_CODE;
    let res = await reqAlchemyPrePay(data);
    if (res.errcode) {
      throw new Error(res.errmsg);
    }
    if (!res.data) {
      throw new Error('No data returned');
    }
    const url = res.data.url;
    if (!url) {
      throw new Error('No url returned');
    }
    console.log('pay url::', url);
    return res.data;
  }

  public async getGasPrice(chainId: number) {
    const key = `gasprice_${chainId}`;
    if (!this.priceMap.has(key)) {
      let price = await new jc.wallet.web3.eth.getGasPrice();
      this.priceMap.set(key, price + '');
    } else {
      setImmediate(async () => {
        let price = await new jc.wallet.web3.eth.getGasPrice();
        this.priceMap.set(key, price + '');
      });
    }
    console.log('gas price', this.priceMap.get(key));
    return this.priceMap.get(key);
  }

  private async updateTokenPrice(chain: string, tokenName: string, env: string) {
    const key = `crypto_usd_${chain}_${tokenName}_${env}`;
    let priceData = await queryTokenUsdPrice(tokenName, chain, env);
    console.log('ETH price data', JSON.stringify(priceData));
    let price = priceData.data.price;
    this.priceMap.set(key, price + '');
  }

  public async queryTokenPrice(chain: string, tokenName: string, env: string = 'release') {
    chain = chain.toUpperCase();
    tokenName = tokenName.toUpperCase();
    const key = `crypto_usd_${chain}_${tokenName}_${env}`;
    if (!this.priceMap.has(key)) {
      try {
        await this.updateTokenPrice(chain, tokenName, env);
      } catch (err) {
        console.log('ETH price err', err);
      }
    } else {
      setImmediate(async () => {
        try {
          await this.updateTokenPrice(chain, tokenName, env);
        } catch (err) {
          console.log('ETH price err', err);
        }
      });
    }
    console.log('ETH price', this.priceMap.get(key));
    return this.priceMap.get(key);
  }

  public async fetchFiatList() {
    return queryFiatList();
  }

  // begin of google pay
  public async queryGoogleProducts(productIds: string[]) {
    let str = await new NativeSvr().queryProducts(productIds.join(','));
    return str;
  }

  public async queryGooglePurchases() {
    let result = await new NativeSvr().queryPurchase();
    let data = JSON.parse(result + '');
    console.log('query purchase result:: ' + data);
    if (data.length === 0) {
      throw new ZError(10, 'no records');
    }
    let res = await verifyGooglePay({ list: data });
    if (res.errcode) {
      throw new ZError(res.errcode, res.errmsg);
    }
    return res.data;
  }

  public async buyGoogleProduct(productId: string, orderId: string) {
    let result = await new NativeSvr().buyProduct(productId, orderId);
    let data = JSON.parse(result + '');
    console.log('native buy result:: ' + data);
    if (data.length === 0) {
      throw new ZError(10, 'no records');
    }
    let res = await verifyGooglePay({ list: data });
    if (res.errcode) {
      throw new ZError(res.errcode, res.errmsg);
    }
    return res.data;
  }

  // end of google pay

  // begin of iOS purchase
  public async queryIOSProducts(productIds: string[]) {
    let str = await new NativeSvr().queryProducts(productIds.join(','));
    return str;
  }

  public async queryIOSPurchases() {
    let result = await new NativeSvr().queryPurchase();
    let data = JSON.parse(result + '');
    console.log('query apple purchase result:: ' + result);
    if (data.length === 0) {
      throw new ZError(10, 'no records');
    }
    let res = await verifyApplePay({ list: data });
    if (res.errcode) {
      throw new ZError(res.errcode, res.errmsg);
    }
    if (res.data.length > 0) {
      for (let i = 0; i < res.data.length; i++) {
        await new NativeSvr().finishTransaction(res.data[i]);
      }
    }
    return data;
  }

  public async beginIOSPurchase(productId: string, orderId: string) {
    let result = await new NativeSvr().buyProduct(productId, orderId);
    let data = JSON.parse(result + '');
    console.log('apple purchase result:: ' + result);
    if (data.length === 0) {
      throw new ZError(10, 'no records');
    }
    let res = await verifyApplePay({ list: data });
    if (res.errcode) {
      throw new ZError(res.errcode, res.errmsg);
    }
    if (res.data.length > 0) {
      for (let i = 0; i < res.data.length; i++) {
        await new NativeSvr().finishTransaction(res.data[i]);
      }
    }
    return data;
  }
  // end of iOS purchase
}
